@@ -1,6 +1,6 @@
 package Module::Build::Base;
 
-# $Id: Base.pm,v 1.17 2002/03/31 12:43:06 ken Exp $
+# $Id: Base.pm,v 1.25 2002/06/26 09:51:13 ken Exp $
 
 use strict;
 use Config;
@@ -48,8 +48,10 @@ sub new {
 		    properties => {
 				   build_script => 'Build',
 				   config_dir => '_build',
-				   prereq => {},
-				   recommended => {},
+				   requires => {},
+				   recommends => {},
+				   build_requires => {},
+				   conflicts => {},
 				   PL_files => {},
 				   %input,
 				   %$cmd_properties,
@@ -57,6 +59,11 @@ sub new {
 		    new_cleanup => {},
 		   }, $package;
 
+  # Synonyms
+  for ($self->{properties}) {
+    $_->{requires} = delete $_->{prereq} if exists $_->{prereq};
+  }
+
   $self->check_manifest;
   $self->check_prereq;
   $self->find_version;
@@ -82,8 +89,8 @@ sub resume {
        module_name
        module_version
        module_version_from
-       prereq
-       recommended
+       requires
+       recommends
        PL_files
        config_dir
        build_script
@@ -136,17 +143,20 @@ EOF
 
 sub find_version {
   my ($self) = @_;
-  return if exists $self->{properties}{module_version};
+  my $p = $self->{properties};
+
+  return if exists $p->{module_version};
   
-  if (exists $self->{properties}{module_version_from}) {
-    my $version = $self->version_from_file($self->{properties}{module_version_from});
-    $self->{properties}{module_version} = $version;
-    delete $self->{properties}{module_version_from};
+  if (exists $p->{module_version_from}) {
+    my $version = $self->version_from_file($p->{module_version_from});
+    $p->{module_version} = $version;
+    delete $p->{module_version_from};
   } else {
     # Try to find the version in 'module_name'
-    my $chief_file = $self->module_name_to_file($self->{properties}{module_name});
-    die "Can't find module '$self->{properties}{module_name}' for version check" unless defined $chief_file;
-    $self->{properties}{module_version} = $self->version_from_file($chief_file);
+    die "No module_name parameter supplied" unless $p->{module_name};
+    my $chief_file = $self->module_name_to_file($p->{module_name});
+    die "Can't find module '$p->{module_name}' for version check" unless defined $chief_file;
+    $p->{module_version} = $self->version_from_file($chief_file);
   }
 }
 
@@ -245,13 +255,13 @@ sub check_prereq {
   my $self = shift;
 
   my $pass = 1;
-  while (my ($modname, $spec) = each %{$self->{properties}{prereq}}) {
+  while (my ($modname, $spec) = each %{$self->{properties}{requires}}) {
     my $thispass = $self->check_installed_version($modname, $spec);
     warn "WARNING: $@\n" unless $thispass;
     $pass &&= $thispass;
   }
 
-  while (my ($modname, $spec) = each %{$self->{properties}{recommended}}) {
+  while (my ($modname, $spec) = each %{$self->{properties}{recommends}}) {
     warn "NOTE: $@\n" unless $self->check_installed_version($modname, $spec);
   }
 
@@ -262,41 +272,61 @@ sub check_prereq {
   return $pass;
 }
 
-# I wish I could set $!, but I can't.
+sub perl_version_to_float {
+  my ($self, $version) = @_;
+  $version =~ s/\./../;
+  $version =~ s/\.(\d+)/sprintf '%03d', $1/eg;
+  return $version;
+}
+
+# I wish I could set $! to a string, but I can't, so I use $@
 sub check_installed_version {
   my ($self, $modname, $spec) = @_;
 
-  my $file = $self->module_name_to_file($modname);
-  unless ($file) {
-    $@ = "Prerequisite $modname isn't installed";
-    return 0;
-  }
+  my $installed_version;
+  if ($modname eq 'perl') {
+    # Check the current perl interpreter
+    # It's much more convenient to use $] here than $^V, but 'man
+    # perlvar' says I'm not supposed to.  Bloody tyrant.
+    $installed_version = $^V ? $self->perl_version_to_float(sprintf "%vd", $^V) : $];
+    
+  } else {
+    my $file = $self->module_name_to_file($modname);
+    unless ($file) {
+      $@ = "Prerequisite $modname isn't installed";
+      return 0;
+    }
 
-  my $version = $self->version_from_file($file);
-  if ($spec and !$version) {
-    $@ = "Couldn't find a \$VERSION in prerequisite '$file'";
-    return 0;
+    $installed_version = $self->version_from_file($file);
+    if ($spec and !$installed_version) {
+      $@ = "Couldn't find a \$VERSION in prerequisite '$file'";
+      return 0;
+    }
   }
-
+  
   my @conditions;
   if ($spec =~ /^\s*([\w.]+)\s*$/) { # A plain number, maybe with dots, letters, and underscores
     @conditions = (">= $spec");
   } else {
-    @conditions = split /\s*,\s*/, $self->{properties}{prereq}{$modname};
+    @conditions = split /\s*,\s*/, $self->{properties}{requires}{$modname};
   }
-
+  
   foreach (@conditions) {
-    if ($_ !~ /^\s*  (<=?|>=?|==|!=)  \s*  [\w.]+  \s*$/x) {
+    unless ( /^\s*  (<=?|>=?|==|!=)  \s*  ([\w.]+)  \s*$/x ) {
       $@ = "Invalid prerequisite condition for $modname: $_";
       return 0;
     }
-    unless (eval "\$version $_") {
-      $@ = "$modname version $version is installed, but we need version $_";
+    if ($modname eq 'perl') {
+      my ($op, $version) = ($1, $2);
+      $_ = "$op " . $self->perl_version_to_float($version);
+    }
+    unless (eval "\$installed_version $_") {
+      $@ = "$modname version $installed_version is installed, but we need version $_";
       return 0;
     }
   }
 
-  return $version ? $version : '0 but true';
+  return $installed_version ? $installed_version : '0 but true';
 }
 
 sub rm_previous_build_script {
@@ -344,6 +374,7 @@ my \$build = resume $build_package (
 );
 eval {\$build->dispatch};
 my \$err = \$@;
+chdir('$build_dir') or die 'Cannot chdir to $build_dir: '.\$!;
 \$build->write_cleanup;  # Always write, even if error occurs
 die \$err if \$err;
 
@@ -610,13 +641,16 @@ sub ACTION_skipcheck {
 sub ACTION_distclean {
   my ($self) = @_;
   
-  $self->depends('realclean');
-  $self->depends('distcheck');
+  $self->depends_on('realclean');
+  $self->depends_on('distcheck');
 }
 
 sub ACTION_distdir {
   my ($self) = @_;
-  
+
+  my $metafile = 'META.yaml';
+  $self->write_metadata($metafile);
+
   my $dist_dir = $self->dist_dir;
   
   require ExtUtils::Manifest;  # ExtUtils::Manifest is not warnings clean.
@@ -624,6 +658,7 @@ sub ACTION_distdir {
   
   my $dist_files = ExtUtils::Manifest::maniread('MANIFEST');
   ExtUtils::Manifest::manicopy($dist_files, $dist_dir, 'best');
+  warn "*** Did you forget to add $metafile to the MANIFEST?\n" unless exists $dist_files->{$metafile};
 }
 
 sub ACTION_disttest {
@@ -655,11 +690,38 @@ sub dist_dir {
   return "$dist_dir-$self->{properties}{module_version}";
 }
 
+sub write_metadata {
+  my ($self, $file) = @_;
+  my $p = $self->{properties};
+
+  $p->{license} ||= 'unknown';
+  unless (grep /^$p->{license}$/, qw(perl gpl restrictive artistic unknown)) {
+    warn "Uknown license type '$p->{license}', setting to 'unknown'\n";
+    $p->{license} = 'unknown';
+  }
+
+  my %metadata = (
+		  distribution_type => 'module',
+		  dynamic_config => 0,
+		  name => $p->{module_name},
+		  version => $p->{module_version},
+		  license => $p->{license},
+		 );
+  
+  foreach (qw(requires build_depends recommends conflicts dynamic_config)) {
+    $metadata{$_} = $p->{$_} if exists $p->{$_};
+  }
+  
+  require YAML;
+  YAML::StoreFile($file, \%metadata);
+}
+
 sub make_tarball {
   my ($self, $dir) = @_;
   
   require Archive::Tar;
   my $files = $self->rscan_dir($dir);
+  
   print "Creating $dir.tar.gz\n";
   Archive::Tar->create_archive("$dir.tar.gz", 1, @$files);
 }
@@ -754,6 +816,20 @@ sub compile_c {
   return $obj_file;
 }
 
+sub link_c {
+  my ($self, $archdir, $file_base) = @_;
+  my $cf = $self->{config}; # For convenience
+
+  my $lib_file = File::Spec->catfile($archdir, File::Basename::basename("$file_base.$cf->{dlext}"));
+  unless ($self->up_to_date("$file_base$cf->{obj_ext}", $lib_file)) {
+    my $linker_flags = $cf->{extra_linker_flags} || '';
+    my $objects = $self->{objects} || [];
+    $self->do_system("$cf->{shrpenv} $cf->{cc} $cf->{lddlflags} -o $lib_file ".
+		     "$file_base$cf->{obj_ext} @$objects $linker_flags")
+      or die "error building $file_base$cf->{obj_ext} from '$file_base.$cf->{dlext}'";
+  }
+}
+
 sub run_perl_script {
   my ($self, $script, $preargs, $postargs) = @_;
   $preargs ||= '';   $postargs ||= '';
@@ -805,14 +881,7 @@ sub process_xs {
   $self->copy_if_modified("$file_base.bs", $archdir, 1);
   
   # .o -> .(a|bundle)
-  my $lib_file = File::Spec->catfile($archdir, File::Basename::basename("$file_base.$cf->{dlext}"));
-  unless ($self->up_to_date("$file_base$cf->{obj_ext}", $lib_file)) {
-    my $linker_flags = $cf->{extra_linker_flags} || '';
-    my $objects = $self->{objects} || [];
-    $self->do_system("$cf->{shrpenv} $cf->{cc} $cf->{lddlflags} -o $lib_file ".
-		     "$file_base$cf->{obj_ext} @$objects $linker_flags")
-      or die "error building $file_base$cf->{obj_ext} from '$file_base.$cf->{dlext}'";
-  }
+  $self->link_c($archdir, $file_base);
 }
 
 sub do_system {
